
/**
 * Use the mergesort algorithm for Objects
 * This sort, like most recursive algorithms, operates with logarithmic efficiency
 */
public class Sort {

	/**
	 * Merges two sorted portions of an array
	 * pre: items[start..mid] is sorted. items[mid+1..end] sorted
	 *  start <= mid <= end
	 * post: items[start..end] is sorted
	 */
	private static void merge(Comparable[] items, int start, int mid, int end){
		//Build a temporary array
		Comparable[] temp = new Comparable[items.length];
		//Establish reference points
		int pos1 = start;
		int pos2 = mid + 1;
		int spot = start;
		
		while(!(pos1 > mid && pos2 > end))	{
			//If the first position is after the middle of the array or if the 
			// second is just before or at the end and the item at pos1 is less than the item at pos2
			// swap items and increment position
			if ((pos1 > mid) || ((pos2 <= end) && (items[pos2].compareTo(items[pos1]) < 0))) {
				temp[spot] = items[pos2];
				pos2 += 1;
			}else{//Swap other items and increment othe position
				temp[spot] = items[pos1];
				pos1 += 1;
			}
			spot += 1;
		}
		//Copy temps back into items
		for (int i = start; i <= end; i++){
			items[i] = temp[i];
		}
	}
		
		/**
		 * Sorts items[start..end]
		 * pre: start > 0, end > 0
		 * post: items[start..end] is sorted low to high
		 */
		public static void mergeSort(Comparable[] items, int start, int end){
			if (start < end){
				//Closely Examine this recursion!!
				int mid = (start + end)/2;
				//Each half is cut in half until each version of items[] has only one item. 
				// Then merge is called
				mergeSort(items, start, mid);
				mergeSort(items, mid+1, end);
				//Once each temp array is cut down to a single element, the merge method can 
				// build a single sorted array
				merge(items, start, mid, end);
		}
	}
}
